#!/bin/sh

# ----------------------------------------------------------------------------
# Clear the screen if it is supported.
# ----------------------------------------------------------------------------
clearscr()
{
	# check if the terminal environment is set up
	[ "$TERM" ] && clear 2> /dev/null
}

# ----------------------------------------------------------------------------
# Print message
# ----------------------------------------------------------------------------
print()
{
	if [ "${OUTPUT_TYPE}" = "NORMAL" ]; then
		echo $* 
	else
		echo $*  >&2
	fi
}

# ----------------------------------------------------------------------------
# Print message
# ----------------------------------------------------------------------------
print_space()
{
	if [ "${OUTPUT_TYPE}" = "NORMAL" ]; then
		echo $* " " 
	else
		echo $* " "  >&2
	fi
}

# ----------------------------------------------------------------------------
# Display error message.
# ----------------------------------------------------------------------------
error()
{
	print ""
	print -e "    Error: $*!" 
	print ""
	return 0
}

# ----------------------------------------------------------------------------
# Pause.
# ----------------------------------------------------------------------------
pause()
{
	[ $# -ne 0 ] && print -e $*
	print -e "Press [Enter] to continue...\c"
	read tmp
	return 0
}

#============================================================
# result
#
#	Write a return code into a file.  We use this
#	since bash2 can only return a number from 0 to 255. 
#
#============================================================
result () {
	echo "$1 " > $RC
}

# ----------------------------------------------------------------------------
# Prompt user for input.
# ----------------------------------------------------------------------------
prompt()
{
	print -n "$* "
	read CMD rest
	return 0
}

# ----------------------------------------------------------------------------
# Get Number 
# ----------------------------------------------------------------------------
get_num()
{
	while prompt "$* "
	do	
		case $CMD in
			[0-9]*)	result $CMD
				return 0
				;;
    			*)	getyn "Wrong number, try it again?" || return 1
				;;
		esac
	done
	return 1
}

# ----------------------------------------------------------------------------
# Get Yes/No
# ----------------------------------------------------------------------------
getyn()
{
	while prompt "$* (y/n)"
	do	case $CMD in
			[yY])	return 0
				;;
			[nN])	return 1
				;;
			*)	print -e "\nPlease answer y or n"
				;;
		esac
	done
}

#---------------------------------------------------------------------------
# initialize all variables
# ----------------------------------------------------------------------------
init_all_variables ()
{
	WAN_HOME=$1/etc/wanpipe
	WAN_BIN_DIR=$1/sbin
	WAN_CONF_DIR=$WAN_HOME
	WAN_SAMPLES_DIR=$WAN_HOME/samples
	WAN_LIB_DIR=$WAN_HOME/lib
	WAN_WANCFG_TEMPL_DIR=$WAN_HOME/wancfg_templates	
	WAN_DOC_DIR=$WAN_HOME/doc
	WAN_INTR_DIR=$WAN_HOME/interfaces
	WAN_MODULE_DIR=/boot/modules
	WAN_FIRMWARE_DIR=$WAN_HOME/firmware
	META_CONF=$WAN_HOME/wanrouter.rc
	ADSL_VCIVPI_LIST=$WAN_HOME/wan_adsl.list
	WAN_SCRIPTS_DIR=$WAN_HOME/scripts
	RC=return_code
	GET_RC="cat $RC"
	WANCFG=$WAN_BIN_DIR/wancfg_legacy
	WANPIPE_CFGFT1=$WAN_BIN_DIR/wanpipe_cfgft1
	WANROUTER_STARTUP=$WAN_BIN_DIR/wanrouter
}

# ----------------------------------------------------------------------------
# Install configuration files.
# ----------------------------------------------------------------------------
install_config()
{

	if [ -d $WAN_CONF_DIR ]; then
		eval \rm -rf $WAN_CONF_DIR.old 2>/dev/null
		eval mv ${WAN_CONF_DIR} ${WAN_CONF_DIR}.old 
	fi
	mkdir $WAN_CONF_DIR

    	print "====> Create WANPIPE Meta configuration file"

	eval "cp -R $1/etc/* ${WAN_CONF_DIR}/"

	# Create meta-configuration file.
	create_metaconf

	# Create file for adsl VCI/VPI standard list.
	create_vcivpi

	# Update wanrouter_rc_file in /etc/rc.conf
	if [ -f /etc/rc.conf ]; then
		. /etc/rc.conf
		if [ -n "$wanrouter_rc_file" ]; then
			cat /etc/rc.conf | awk '{ gsub("^wanrouter_rc_file.*", "") ; print }' > tmp.$$
			cat tmp.$$				> /etc/rc.conf
			rm -f tmp.$$
		fi
		echo "wanrouter_rc_file=\"${META_CONF}\""	>> /etc/rc.conf
	else
		print ""
		print "WARNING:"
		print -e "\tFailed to update /etc/rc.conf file!"
		print -e "\tPlease add manually the following line into /etc/rc.conf:"
		print -e "\t\twanrouter_rc_file=\"${META_CONF}\""
		print "" 
	fi 

    	return 0
}

# ----------------------------------------------------------------------------
# Create meta-configuration file.
# ----------------------------------------------------------------------------
create_metaconf()
{
	if [ -f $META_CONF ]; then
		. $META_CONF
	fi

	# Select directory for the log file.
	WAN_LOG=/var/log/wanrouter
	if [ ! -d /var/log ]; then	
		mkdir /var/log
	fi

	# Select directory for the lock file.
	WAN_LOCK_DIR=/var/lock
	if [ ! -d $WAN_LOCK_DIR ]; then   
		mkdir $WAN_LOCK_DIR
	fi

	# Create start/stop script directory
	if [ ! -d $WAN_SCRIPTS_DIR ]; then   
		mkdir $WAN_SCRIPTS_DIR
	fi

	if [ "$WAN_DEVICES" = "" ]; then
		WAN_DEVICES="wanpipe1"
	fi 

	cat > $META_CONF << ENDOFTEXT

# $META_CONF	WAN router meta-configuration file.
#
#		This file defines variables used by the router shell scripts
#		and should be located in $WAN_CONF_DIR directory.  These are:
#
#               ROUTER_BOOT=    	Boot flag (YES/NO).
#               WAN_CONF_DIR=   	Directory of wanpipe config files.
#		WAN_INTR_DIR=		Where to put wanpipe interface files.
#		WAN_FIRMWARE_DIR=	Where to put protocol firmware files.
#               WAN_LOG=       		Where to put start-up log file.
#               WAN_LOCK_DIR=   	Directory of wanpipe device lock files.
#               WAN_DEVICES=    	Name of the wanpipe devices to be
#                               	loaded on 'wanrouter start'
#                               	(ex: "wanpipe1 wanpipe2 wanpipe3...")
#
#                    	Note: Name of wanpipe devices correspond
#                             to the configuration files in
#                             $WAN_CONF_DIR directory:
#                             (ex. $WAN_CONF_DIR/wanpipe1.conf )
#
#               Note:   This file is 'executed' by the shell script, so
#                       the usual shell syntax must be observed.        
ENDOFTEXT

	echo "ROUTER_BOOT=YES"				>> $META_CONF
	echo "WAN_BIN_DIR=$WAN_BIN_DIR" 		>> $META_CONF
	echo "WAN_CONF_DIR=$WAN_CONF_DIR" 		>> $META_CONF
	echo "WAN_INTR_DIR=$WAN_INTR_DIR" 		>> $META_CONF
	echo "WAN_MODULE_DIR=$WAN_MODULE_DIR" 		>> $META_CONF
	echo "WAN_FIRMWARE_DIR=$WAN_FIRMWARE_DIR" 	>> $META_CONF
	echo "WAN_LOG=$WAN_LOG"				>> $META_CONF
	echo "WAN_LOCK_DIR=$WAN_LOCK_DIR"		>> $META_CONF
	echo "WAN_LIB_DIR=$WAN_LIB_DIR" 		>> $META_CONF
	echo "WAN_ADSL_LIST=$ADSL_VCIVPI_LIST" 		>> $META_CONF
	echo "WAN_SCRIPTS_DIR=$WAN_SCRIPTS_DIR" 	>> $META_CONF
	echo "ZAPTEL_BIN_DIR=$ZAPTEL_BIN_DIR" 	>> $META_CONF
	echo "WAN_DEVICES=\"$WAN_DEVICES\""		>> $META_CONF

	return 0
}

# ----------------------------------------------------------------------------
# Create file for VCI/VPI standard list.
# ----------------------------------------------------------------------------
create_vcivpi()
{

	[ -f $ADSL_VCIVPI_LIST ] &&  mv $ADSL_VCIVPI_LIST $ADSL_VCIVPI_LIST.old 

	cat > $ADSL_VCIVPI_LIST << ENDOFTEXT

# $ADSL_VCIVPI_LIST	WAN ADSL VCI/VPI standard list.
#
#		This file defines standard list for VCI/VPI 
#		values for ADSL connection and should be 
#		located in $WAN_CONF_DIR directory.  
#		These are:
#
#		VCI	VPI
#
#
#               Note:   If the new VCI/VPI values will be found, 
#			this list will be updated automatically 
#			during shutting down ADSL device.
# 
ENDOFTEXT

	echo "39 0"	>> $ADSL_VCIVPI_LIST
	echo "16 0"	>> $ADSL_VCIVPI_LIST
	echo "38 0"	>> $ADSL_VCIVPI_LIST
	echo "100 0"	>> $ADSL_VCIVPI_LIST
	echo "35 0"	>> $ADSL_VCIVPI_LIST
	echo "35 8"	>> $ADSL_VCIVPI_LIST

	return 0
}


###############################################################################
##
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:/usr/pkg/bin
DATE=`date`
OSYSTEM=`uname -s`
PLATFORM=`uname -m`
OSYSTEM_VER=`uname -r`
CURDIR=`pwd`
PKG_NAME=$1
PKG_ACTION=$2
WAN_BASE=$3
WAN_PKGDIR=$4/$PKG_NAME
WAN_BIN_DIR=${WAN_BASE}/sbin
ZAPTEL_BIN_DIR=${WAN_BASE}/bin

OUTPUT_TYPE=NORMAL

init_all_variables ${WAN_BASE}
install_config $WAN_PKGDIR || return 1

eval "cp $WAN_PKGDIR/scripts/wanpipe ${WAN_BASE}/etc/rc.d/"
eval "cp $WAN_PKGDIR/scripts/wancfg_legacy ${WAN_BIN_DIR}/"
eval "cp $WAN_PKGDIR/scripts/wancfg_zaptel ${WAN_BIN_DIR}/"
eval "cp $WAN_PKGDIR/scripts/wanrouter ${WAN_BIN_DIR}/"
eval "cp $WAN_PKGDIR/scripts/wanpipe_cfgft1 ${WAN_BIN_DIR}/"

return 0
