/*
 * Copyright (c) 1997, 1998, 1999
 *	Alex Feldman <al.feldman@sangoma.com>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Alex Feldman.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alex Feldman AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Alex Feldman OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: sdla_bsd.c,v 1.3 2007-12-24 18:23:47 sangoma Exp $
 */

/*****************************************************************************
 * sdla_bsd.c	SDLA DEV Support Module.  Main module.
 *
 *		This module is a compeled with Wanpipe sources.
 *
 * Author:	Alex Feldman	<al.feldman@sangoma.com>
 ****************************************************************************
 */

#undef INTR_TEST	/* uncomment this to check that driver RX INTR */
#undef DEBUG

#include <sys/types.h>
#include <sys/param.h>
#if (__FreeBSD_version < 500000)
# include "sdla.h"		/* number of Sangoma cards */
# include <stddef.h>
#else
# define NSDLA		20	/* temporary value */
#endif
#include <sys/systm.h>
#include <sys/syslog.h>
#include <sys/errno.h>
#include <sys/mbuf.h>
#include <sys/proc.h>
#include <sys/sockio.h>
#include <sys/socket.h>
#include <sys/kernel.h>
#include <sys/module.h>
#include <sys/sysent.h>
#if (__FreeBSD_version > 400000)
# include <sys/bus.h>
#endif
#include <sys/interrupt.h>
#include <machine/bus.h>
#if (__FreeBSD_version > 400000)
# include <sys/rman.h>
# include <machine/resource.h>
#endif
#include <vm/vm.h>
#include <vm/pmap.h>

#include <sdlapci.h>
#include <sdlasfm.h>
#include <sdla_bsd.h>
 
#if (__FreeBSD_version <= 400000)
# device_get_name(dev)	SDLA_DEV_NAME	
#endif

/****** Global Data *******************************************************/
int    	Sangoma_cards_no = 0;
int    	Sangoma_devices_no = 0;
int	Sangoma_PCI_cards_no = 0; 

/*************************************************************************/
/* SDLA PCI device relative entry point 	 	 		 */

static int	sdla_pci_probe	__P((device_t));
static int	sdla_pci_attach	__P((device_t));
static int	sdla_pci_detach	__P((device_t));
driver_intr_t	sdla_pci_intr;

static device_method_t sdla_pci_methods[] = {
	DEVMETHOD(device_probe,		sdla_pci_probe),
	DEVMETHOD(device_attach,	sdla_pci_attach),
	DEVMETHOD(device_detach,	sdla_pci_detach),
	{0, 0}
};

static driver_t	sdla_pci_driver = {
	SDLA_DEV_NAME,
	sdla_pci_methods,
	sizeof(struct sdla_softc),
};

static devclass_t sdla_devclass;

DRIVER_MODULE(sdla, pci, sdla_pci_driver, sdla_devclass, 0, 0);
/***********************************************************************/
int sdla_alloc_memory(device_t dev, int size);
int sdla_alloc_irq(device_t dev, int flags);
void sdla_release_resources(device_t dev);


/******* PCI Device Driver Entry Points ********************************/
static int sdla_supported(device_t dev)
{
    	u_int16_t	vendor = pci_get_vendor(dev);
    	u_int16_t	subvendor = pci_get_subvendor(dev);
    	u_int16_t	device = pci_get_device(dev);

	switch(vendor){
	case V3_VENDOR_ID:
		if (subvendor == SANGOMA_SUBSYS_VENDOR){
			return 0;
		}
		break;
	case PCI_VENDOR_ID_GSI:
		if (device == PCI_DEVICE_ID_GSI_PULSAR){
			return 0;
		}
		break;
	case SANGOMA_PCI_VENDOR_OLD:
		if (device == SANGOMA_PCI_DEVICE){
			return 0;
		}
		break;
	case SANGOMA_PCI_VENDOR:
		switch(device){
		case SANGOMA_PCI_DEVICE:
		case SANGOMA_PCI_4_DEVICE:
			return 0;
		default:
			return 0;	/* our new cards */
		}
		break;
	}
	return -EINVAL;
}

static int sdla_pci_probe(device_t dev)
{
    	u_int16_t	vendor = pci_get_vendor(dev);
    	u_int16_t	subvendor = pci_get_subvendor(dev);
    	u_int16_t	subdevice = pci_get_subdevice(dev);
    	u_int16_t	adapter_type = 0;

    	/* Check ivendor and subvendor ID */
	if (sdla_supported(dev)){
#if defined(DEBUG)
		printf("%s: not our card (vendor=%04x, subvendor=%04x)!\n",
					device_get_name(dev),vendor,subvendor);
#endif
		return ENXIO;
    	}

	switch(vendor){
	case V3_VENDOR_ID:
		adapter_type = subdevice & 0xFF;
		switch(adapter_type){
		case S5141_ADPTR_1_CPU_SERIAL:
    			device_set_desc(dev, "Sangoma S5141/FT1 (Single CPU) adapter");
			break;
		case S5142_ADPTR_2_CPU_SERIAL:
    			device_set_desc(dev, "Sangoma S5142 (Dual CPU) adapter");
			break;
		case S5143_ADPTR_1_CPU_FT1:
    			device_set_desc(dev, "Sangoma S5143 (Single CPU) FT1 adapter");
			break;
		case S5144_ADPTR_1_CPU_T1E1:
		case S5148_ADPTR_1_CPU_T1E1:
    			device_set_desc(dev, "Sangoma S5144 (Single CPU) T1/E1 adapter");
			break;
		case S5145_ADPTR_1_CPU_56K:		
    			device_set_desc(dev, "Sangoma S5145 (Single CPU) 56K adapter");
			break;
		case S5147_ADPTR_2_CPU_T1E1:
    			device_set_desc(dev, "Sangoma S5147 (Dual CPU) T1/E1 adapter");
			break;
		}
		break;

	case PCI_VENDOR_ID_GSI:
    		device_set_desc(dev, "Sangoma S518 ADSL adapter");
		break;

	case SANGOMA_PCI_VENDOR:
	case SANGOMA_PCI_VENDOR_OLD:
		switch(subvendor){
		case A101_1TE1_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A101 T1/E1 adapter");
			break;
		case A101_2TE1_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A102 T1/E1 adapter");
			break;
		case A104_4TE1_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A104 T1/E1 adapter");
			break;
		case A300_UTE3_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A301 T3/E3 adapter");
			break;
		case AFT_1TE1_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A101d T1/E1 adapter");
			break;
		case AFT_2TE1_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A102d T1/E1 adapter");
			break;
		case AFT_4TE1_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A104d T1/E1 adapter");
			break;
		case AFT_8TE1_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A108d T1/E1 adapter");
			break;
		case A200_REMORA_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A200 Remora adapter");
			break;
		case A400_REMORA_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A400 Remora adapter");
			break;
		case A300_UTE3_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A301 T3/E3 adapter");
			break;
		case A305_CTE3_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A305 T3/E3 adapter");
			break;
		case AFT_ISDN_BRI_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A500 ISDN BRI adapter");
			break;
		case AFT_56K_SHARK_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A056 56K adapter");
			break;
		case AFT_2SERIAL_V35X21_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A142 V.35/X.21 adapter");
			break;
		case AFT_4SERIAL_V35X21_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A144 V.35/X.21 adapter");
			break;
		case AFT_2SERIAL_RS232_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A142 RS232 adapter");
			break;
		case AFT_4SERIAL_RS232_SUBSYS_VENDOR:
    			device_set_desc(dev, "Sangoma AFT-A144 RS232 adapter");
			break;
		}
	}	
    	return 0;
}

static int sdla_pci_attach(device_t dev)
{
    	u_int16_t		vendor = pci_get_vendor(dev);
    	u_int16_t		subvendor = pci_get_subvendor(dev);
	u_int16_t		subsystem_id = pci_read_config(dev, PCI_SUBSYS_ID_WORD, 2);
	u_int16_t		adapter_type, adapter_subtype = 0x0;
    	int			type = 0, dual_cpu = 0, line_no = 1;
	int			error;

    	/* Check ivendor and subvendor ID */
	adapter_type = subsystem_id;
	switch(vendor){
	case V3_VENDOR_ID:
		type = SDLA_S514;
		if (adapter_type == S5148_ADPTR_1_CPU_T1E1){
			adapter_type = S5144_ADPTR_1_CPU_T1E1;
		}
    		/* Check number of CPUs */
		switch(adapter_type){
		case S5142_ADPTR_2_CPU_SERIAL:
		case S5147_ADPTR_2_CPU_T1E1:
			dual_cpu = 1;
			break;
		}
		break;
	case PCI_VENDOR_ID_GSI:
		type = SDLA_ADSL;
		adapter_type = S518_ADPTR_1_CPU_ADSL;
		break;
	case SANGOMA_PCI_VENDOR_OLD:
	case SANGOMA_PCI_VENDOR:
		type = SDLA_AFT;
		switch(subvendor){
		case A101_1TE1_SUBSYS_VENDOR:
			adapter_type	= A101_ADPTR_1TE1;
			break;
		case A101_2TE1_SUBSYS_VENDOR:
			dual_cpu = 1;
			adapter_type	= A101_ADPTR_2TE1;
			break;
		case A104_4TE1_SUBSYS_VENDOR:
			adapter_type	= A104_ADPTR_4TE1;
			line_no	= 4;
			break;
		case A300_UTE3_SUBSYS_VENDOR:
			adapter_type	= A300_ADPTR_U_1TE3;
			break;
		case AFT_1TE1_SHARK_SUBSYS_VENDOR:
			adapter_type	= A101_ADPTR_1TE1;
			adapter_subtype = AFT_SUBTYPE_SHARK;
			break;
		case AFT_2TE1_SHARK_SUBSYS_VENDOR:
			adapter_type	= A101_ADPTR_2TE1;
			adapter_subtype = AFT_SUBTYPE_SHARK;
			line_no	= 2;
			break;
		case AFT_4TE1_SHARK_SUBSYS_VENDOR:
			adapter_type	= A104_ADPTR_4TE1;
			adapter_subtype = AFT_SUBTYPE_SHARK;
			line_no	= 4;
			break;
		case AFT_8TE1_SHARK_SUBSYS_VENDOR:
			adapter_type	= A108_ADPTR_8TE1;
			adapter_subtype = AFT_SUBTYPE_SHARK;
			line_no	= 8;
			break;
		case A200_REMORA_SHARK_SUBSYS_VENDOR:
			adapter_type	= A200_ADPTR_ANALOG;
			adapter_subtype = AFT_SUBTYPE_SHARK;
			break;
		case A400_REMORA_SHARK_SUBSYS_VENDOR:
			adapter_type	= A400_ADPTR_ANALOG;
			adapter_subtype = AFT_SUBTYPE_SHARK;
			break;
		case A300_UTE3_SHARK_SUBSYS_VENDOR:
			adapter_type	= A300_ADPTR_U_1TE3;
			adapter_subtype	= AFT_SUBTYPE_SHARK;
			break;
		case A305_CTE3_SHARK_SUBSYS_VENDOR:
			adapter_type	= A305_ADPTR_C_1TE3;
			adapter_subtype	= AFT_SUBTYPE_SHARK;
			break;
		case AFT_ISDN_BRI_SHARK_SUBSYS_VENDOR:
			adapter_type	= AFT_ADPTR_ISDN;
			adapter_subtype	= AFT_SUBTYPE_SHARK;
			line_no	= 24;
			break;
		case AFT_56K_SHARK_SUBSYS_VENDOR:
			adapter_type	= AFT_ADPTR_56K;
			adapter_subtype	= AFT_SUBTYPE_SHARK;
			break;
		default:
#if defined(DEBUG)
			printf("%s: Unsupported subvendor id (0x%04x)!\n",
					device_get_name(dev), subvendor);
#endif
			return ENXIO;
		}
		break;
	default:
#if defined(DEBUG)
		printf("%s: not our card (vendor=%04x,subvendor=%04x)!\n",
					device_get_name(dev),vendor,subvendor);
#endif
		return ENXIO;
	}

    	/* Allocate irq for both CPU's */
    	error = sdla_alloc_irq(dev, RF_SHAREABLE);
    	if (error){
		printf("%s: Failed allocate shared IRQ for Sangoma PCI device!\n",
				device_get_name(dev));
		sdla_release_resources(dev);
		return (error);
    	}

    	/* Increment number of SDLA adapters, include CPU_A and CPU_B */
    	Sangoma_cards_no += (dual_cpu) ? 2 : 1;
    	/* Incremenet number of S514 cards */
    	Sangoma_PCI_cards_no++;
	/* Increment number of Sangoma devices */
	switch(adapter_type){
	case S5141_ADPTR_1_CPU_SERIAL:
		Sangoma_devices_no += 2;
		break;
	case S5142_ADPTR_2_CPU_SERIAL:
		Sangoma_devices_no += 4;
		break;
	case S5143_ADPTR_1_CPU_FT1:
	case S5144_ADPTR_1_CPU_T1E1:
	/*case S5148_ADPTR_1_CPU_T1E1: The same value as _ADSL */
	case S5145_ADPTR_1_CPU_56K:		
	case S518_ADPTR_1_CPU_ADSL: 
	case A101_ADPTR_1TE1:
	case A200_ADPTR_ANALOG:
	case A400_ADPTR_ANALOG:
	case A300_ADPTR_U_1TE3:
	case A305_ADPTR_C_1TE3:
	case AFT_ADPTR_56K:
		Sangoma_devices_no ++;
		break;
	case A101_ADPTR_2TE1:
		Sangoma_devices_no += 2;
		break;
	case A104_ADPTR_4TE1:
		Sangoma_devices_no += 4;
		break;
	case A108_ADPTR_8TE1:
		Sangoma_devices_no += 8;
		break;
	case S5147_ADPTR_2_CPU_T1E1:
		Sangoma_devices_no += 2;
		break;
	case AFT_ADPTR_ISDN:
		Sangoma_devices_no += 24;
		break;
	}

    	return 0;
}

static int sdla_pci_detach(device_t dev)
{
	sdla_release_resources(dev);
	return 0;
}
/****************************************************************************/
/*		Common resource functions				    */
/****************************************************************************/
/*
 * Allocate an irq resource with the given resource id.
 */
int sdla_alloc_irq(device_t dev, int flags)
{
    	struct sdla_softc*	sc = device_get_softc(dev);

    	sc->irq_res = 
		bus_alloc_resource(
			dev, SYS_RES_IRQ, 
			&sc->irq_rid, 0ul, ~0ul, 1, (RF_ACTIVE | flags));
    	if (!sc->irq_res){
		sc->irq_rid = 0;
		return ENOENT;		
	}
	return 0;
}

/*
 * Release all resources
 */
void sdla_release_resources(device_t dev)
{
    	struct sdla_softc*  sc = device_get_softc(dev);

    	if (sc->irq_res){
		bus_release_resource(	dev, SYS_RES_IRQ, 
					sc->irq_rid, sc->irq_res);
		sc->irq_res = 0;
		sc->irq_rid = 0;
    	}
	return;
}
